class EthereumError(Exception):
    pass


class BadResponseError(EthereumError):

    def __init__(self, msg, code, *args, **kwargs):
        super().__init__(msg, code, *args, **kwargs)
        self.msg = msg
        self.code = code


class BadStatusError(EthereumError):
    pass


class BadJsonError(EthereumError):
    pass



import logging
import asyncio
import warnings
import abc
from urllib.parse import urlparse

try:
    from abc import ABC
except ImportError:
    class ABC(metaclass=abc.ABCMeta):
        pass

import aiohttp
import async_timeout

try:
    import uvloop
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
except ImportError:
    pass

try:
    import ujson as json  # noqa
except ImportError:
    import json

from .errors import BadResponseError, BadStatusError, BadJsonError
from .management import RpcMixin


logger = logging.getLogger('asyncio_client')


_reconnect_times = 3


class BaseAsyncIOClient(ABC):
    """Abstract class for creating client.
    """

    @abc.abstractmethod
    def rpc_call(self, method, params=None, id_=None):
        """Implements RPC 2.0 call to node server.

        :param method: RPC node method
        :type method: str

        :param params: Input params for method
        :type params: list

        :param id_: ID of current request
        :type id_: str or int

        :return: `Response from RPC`
        """

    @asyncio.coroutine
    def _call(self, method, params=None, id_=None):
        warnings.warn('This method is deprecated and will be removed '
                      'in v0.3.0', DeprecationWarning)
        return (yield from self.rpc_call(method, params, id_))


class AsyncIOHTTPClient(BaseAsyncIOClient, RpcMixin):
    """Creates AsyncIOHTTPClient client to communicate via HTTP(s).

    :param host: Host on ethereum node
    :type host: str

    :param port: Port on ethereum node
    :type port: int

    :param tls: Use SSL connection
    :type tls: bool

    :param timeout: Total time of timeout call
    :type timeout: int

    :param loop: An optional *event loop* instance
                 (uses :func:`asyncio.get_event_loop` if not specified).
    :type loop: :ref:`EventLoop<asyncio-event-loop>`

    :return: :class:`AsyncIOHTTPClient` instance.
    """

    def __init__(self, host='127.0.0.1', port=8545, tls=False,
                 timeout=60, *, loop=None):
        self._host = host
        self._port = port
        self._tls = tls
        self._timeout = timeout
        self._id = 1
        self._loop = loop or asyncio.get_event_loop()

    @property
    def _endpoint(self):
        scheme = 'http'
        if self._tls:
            scheme += 's'
        return '{0}://{1}:{2}'.format(scheme, self._host, self._port)

    @asyncio.coroutine
    def rpc_call(self, method, params=None, id_=None):
        params = params or []
        data = {
            'jsonrpc': '2.0',
            'method': method,
            'params': params,
            'id': id_ or self._id,
        }
        try:
            with aiohttp.ClientSession(loop=self._loop) as session:
                with async_timeout.timeout(self._timeout,
                                           loop=self._loop):
                    r = yield from session.post(
                        url=self._endpoint,
                        data=json.dumps(data),
                        headers={'Content-Type': 'application/json'}
                    )
        except aiohttp.ClientConnectorError as e:
            raise ConnectionError(e)

        if r.status != 200:
            raise BadStatusError(r.status)

        try:
            response = yield from r.json(loads=json.loads)
        except ValueError:
            raise BadJsonError('Invalid received json from node.')

        if not id_:
            self._id += 1
        try:
            return response['result']
        except KeyError:
            raise BadResponseError(response['error']['message'],
                                   response['error']['code'])


class AsyncIOIPCClient(BaseAsyncIOClient, RpcMixin):
    """Creates AsyncIOIPCClient client to communicate via IPC.

    :param reader: Instance of the stream reader
    :type reader: :class:`asyncio.streams.StreamReader`

    :param writer: Instance of the stream writer
    :type writer: :class:`asyncio.streams.StreamWriter`

    :param unix_path: Unix domain path
    :type unix_path: str

    :param timeout: An optional total time of timeout call
    :type timeout: int

    :param loop: An optional *event loop* instance
                 (uses :func:`asyncio.get_event_loop` if not specified).
    :type loop: :ref:`EventLoop<asyncio-event-loop>`

    :return: :class:`AsyncIOIPCClient` instance.
    """

    def __init__(self, reader, writer, unix_path, timeout=60, *, loop=None):
        self._reader = reader
        self._writer = writer
        self._timeout = timeout
        self._unix_path = unix_path
        self._id = 1
        self._loop = loop or asyncio.get_event_loop()
        self._lock = asyncio.Semaphore(1, loop=self._loop)

    @asyncio.coroutine
    def rpc_call(self, method, params=None, id_=None):
        params = params or []
        data = {
            'jsonrpc': '2.0',
            'method': method,
            'params': params,
            'id': id_ or self._id,
        }
        with (yield from self._lock):
            self._writer.write(json.dumps(data).encode('utf-8'))
            b = yield from self._reader.readline()
            if not b:
                tried = 0
                while True:
                    try:
                        new_client = yield from create_ethereum_client(
                            self._unix_path, self._timeout)
                    except Exception:
                        tried += 1
                        if tried == _reconnect_times:
                            raise ConnectionError('Didn\'t receive any data, '
                                                  'connection refused.')
                    else:
                        break
                self._id = new_client._id
                self._reader = new_client._reader
                self._writer = new_client._writer
                self._lock = new_client._lock
                return (yield from self.rpc_call(method, params, id_))

        try:
            response = json.loads(b.decode('utf-8'))
        except ValueError:
            raise BadJsonError('Invalid received json from node.')

        if not id_:
            self._id += 1
        try:
            return response['result']
        except KeyError:
            raise BadResponseError(response['error']['message'],
                                   response['error']['code'])


@asyncio.coroutine
def create_ethereum_client(uri, timeout=60, *, loop=None):
    """Create client to ethereum node based on schema.

    :param uri: Host on ethereum node
    :type uri: str

    :param timeout: An optional total time of timeout call
    :type timeout: int

    :param loop: An optional *event loop* instance
                 (uses :func:`asyncio.get_event_loop` if not specified).
    :type loop: :ref:`EventLoop<asyncio-event-loop>`

    :return: :class:`BaseAsyncIOClient` instance.
    """
    if loop is None:
        loop = asyncio.get_event_loop()

    presult = urlparse(uri)
    if presult.scheme in ('ipc', 'unix'):
        reader, writer = yield from asyncio.open_unix_connection(presult.path,
                                                                 loop=loop)
        return AsyncIOIPCClient(reader, writer, uri, timeout, loop=loop)
    elif presult.scheme in ('http', 'https'):
        tls = False
        if presult.scheme[-1] == 's':
            tls = True
        netloc = presult.netloc.split(':')
        if len(netloc) == 1:
            host, port = netloc[0], 80
        else:
            host, port = netloc
        return AsyncIOHTTPClient(host, port, tls, timeout, loop=loop)
    else:
        raise RuntimeError('This scheme does not supported.')



import warnings
import asyncio

from ..utils import add_0x


class DbMixin:

    @asyncio.coroutine
    def db_putString(self, db_name, key, value):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#db_putstring

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('db_putString',
                                         [db_name, key, value]))

    @asyncio.coroutine
    def db_getString(self, db_name, key):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#db_getstring

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('db_getString', [db_name, key]))

    @asyncio.coroutine
    def db_putHex(self, db_name, key, value):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#db_puthex

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        if not value.startswith('0x'):
            value = add_0x(value)
        return (yield from self.rpc_call('db_putHex', [db_name, key, value]))

    def db_getHex(self, db_name, key):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#db_gethex

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('db_getHex', [db_name, key]))



import asyncio


class AdminMixin:

    @asyncio.coroutine
    def admin_addPeer(self, url):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_addpeer

        :param url: Enode url of peer
        :type url: str

        :rtype: bool
        """
        return (yield from self.rpc_call('admin_addPeer', [url]))

    @asyncio.coroutine
    def admin_datadir(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_datadir

        :return: path
        :rtype: str
        """
        return (yield from self.rpc_call('admin_datadir'))

    @asyncio.coroutine
    def admin_nodeInfo(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_nodeinfo

        :return: info
        :rtype: dict
        """
        return (yield from self.rpc_call('admin_nodeInfo'))

    @asyncio.coroutine
    def admin_peers(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_peers

        :return: info
        :rtype: list
        """
        return (yield from self.rpc_call('admin_peers'))

    @asyncio.coroutine
    def admin_setSolc(self, path='/usr/bin/solc'):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_setsolc

        NOT AVAILABLE
        """
        return (yield from self.rpc_call('admin_setSolc', [path]))

    @asyncio.coroutine
    def admin_startRPC(self, host='localhost', port=8545, cors=None, apis=None):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_startrpc

        :param host: Network interface to open the listener socket (optional)
        :type host: str

        :param port: Network port to open the listener socket (optional)
        :type port: int

        :param cors: Cross-origin resource sharing header to use (optional)
        :type cors: str

        :param apis: API modules to offer over this interface (optional)
        :type apis: str

        :rtype: bool
        """
        if cors is None:
            cors = []
        if apis is None:
            apis = ['eth', 'net', 'web3']

        return (yield from self.rpc_call('admin_startRPC',
                                         [host, port,
                                          ','.join(cors), ','.join(apis)]))

    @asyncio.coroutine
    def admin_startWS(self, host='localhost', port=8546, cors=None, apis=None):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_startws

        :param host: Network interface to open the listener socket (optional)
        :type host: str

        :param port: Network port to open the listener socket (optional)
        :type port: int

        :param cors: Cross-origin resource sharing header to use (optional)
        :type cors: str

        :param apis: API modules to offer over this interface (optional)
        :type apis: str

        :rtype: bool
        """
        if cors is None:
            cors = []
        if apis is None:
            apis = ['eth', 'net', 'web3']

        return (yield from self.rpc_call('admin_startWS',
                                         [host, port,
                                          ','.join(cors), ','.join(apis)]))

    @asyncio.coroutine
    def admin_stopRPC(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_stoprpc

        :rtype: bool
        """
        return (yield from self.rpc_call('admin_stopRPC'))

    @asyncio.coroutine
    def admin_stopWS(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#admin_stopws

        :rtype: bool
        """
        return (yield from self.rpc_call('admin_stopWS'))



from .admin import AdminMixin
from .db import DbMixin
from .debug import DebugMixin
from .eth import EthMixin
from .miner import MinerMixin
from .net import NetMixin
from .personal import PersonalMixin
from .shh import ShhMixin
from .txpool import TxpoolMixin
from .web3 import Web3Mixin


__all__ = [
    'AdminMixin',
    'DbMixin',
    'DebugMixin',
    'EthMixin',
    'MinerMixin',
    'NetMixin',
    'PersonalMixin',
    'ShhMixin',
    'TxpoolMixin',
    'Web3Mixin',
    'RpcMixin',
]


class RpcMixin(AdminMixin, DbMixin, DebugMixin, EthMixin, MinerMixin,
               NetMixin, PersonalMixin, ShhMixin, TxpoolMixin, Web3Mixin):
    pass



import asyncio

from ..utils import ether_to_wei


class PersonalMixin:

    @asyncio.coroutine
    def personal_importRawKey(self, keydata, passphrase):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_importrawkey

        :param keydata: Unencrypted private key
        :type keydata: str

        :param passphrase: Passphrase of private key
        :type passphrase: str

        :return: address
        :rtype: hex
        """
        return (yield from self.rpc_call('personal_importRawKey',
                                         [keydata, passphrase]))

    @asyncio.coroutine
    def personal_listAccounts(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_listaccounts

        :return: addresses
        :rtype: list
        """
        return (yield from self.rpc_call('personal_listAccounts'))

    @asyncio.coroutine
    def personal_lockAccount(self, address):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_lockaccount

        :param address: Account address
        :type address: str

        :rtype: bool
        """
        return (yield from self.rpc_call('personal_lockAccount', [address]))

    @asyncio.coroutine
    def personal_newAccount(self, passphrase=None):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_newaccount

        :param passphrase: Passphrase of account (optional)
        :type passphrase: str

        :return: address
        :rtype: str
        """
        return (yield from self.rpc_call('personal_newAccount', [passphrase]))

    @asyncio.coroutine
    def personal_unlockAccount(self, address, passphrase=None, duration=None):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_unlockaccount

        :param address: Account address
        :type address: str

        :param passphrase: Passphrase of account (optional)
        :type passphrase: str

        :param duration: Duration to be unlocked (optional)
        :type duration: int

        :rtype: bool
        """
        return (yield from self.rpc_call('personal_unlockAccount',
                                         [address, passphrase, duration]))

    @asyncio.coroutine
    def personal_sendTransaction(self, from_, to=None, gas=None,
                                 gas_price=None, value=None, data=None,
                                 nonce=None, passphrase=None):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_sendtransaction

        :param from_: From account address
        :type from_: str

        :param to: To account address (optional)
        :type to: str

        :param gas: Gas amount for current transaction (optional)
        :type gas: int

        :param gas_price: Gas price for current transaction (optional)
        :type gas_price: int

        :param value: Amount of ether to send (optional)
        :type value: int

        :param data: Additional data for transaction (optional)
        :type data: hex

        :param nonce: Unique nonce for transaction (optional)
        :type nonce: int

        :param passphrase: Passphrase of account (optional)
        :type passphrase: str

        :return: txhash
        :rtype: str
        """
        obj = {}
        obj['from'] = from_
        if to is not None:
            obj['to'] = to
        if gas is not None:
            obj['gas'] = hex(gas)
        if gas_price is not None:
            obj['gasPrice'] = hex(gas_price)
        if value is not None:
            obj['value'] = hex(ether_to_wei(value))
        if data is not None:
            obj['data'] = data
        if nonce is not None:
            obj['nonce'] = hex(nonce)

        return (yield from self.rpc_call('personal_sendTransaction',
                                         [obj, passphrase]))

    @asyncio.coroutine
    def personal_sign(self, message, account, password=None):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_sign

        :param message: Message for sign
        :type message: str

        :param account: Account address
        :type account: str

        :param password: Password of account (optional)
        :type password: str

        :return: signature
        :rtype: str
        """
        return (yield from self.rpc_call('personal_sign',
                                         [message, account, password]))

    @asyncio.coroutine
    def personal_ecRecover(self, message, signature):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#personal_ecrecover

        :param message: Message for sign
        :type message: str

        :param password: Signature of account (optional)
        :type password: str

        :return: address
        :rtype: str
        """
        return (yield from self.rpc_call('personal_ecRecover',
                                         [message, signature]))



from .constants import BLOCK_TAGS


def add_0x(string):
    """Add 0x to string at start.
    """
    if isinstance(string, bytes):
        string = string.decode('utf-8')
    return '0x' + str(string)


def hex_to_dec(x):
    """Convert hex to decimal
    """
    return int(x, 16)


def wei_to_ether(wei):
    """Convert wei to ether
    """
    return 1.0 * wei / 10**18


def gwei_to_ether(wei):
    """Convert gwei to ether
    """
    return 1.0 * wei / 10**9


def ether_to_wei(ether):
    """Convert ether to wei
    """
    return int(ether * 10**18)


def ether_to_gwei(ether):
    """Convert ether to Gwei
    """
    return int(ether * 10**9)


def validate_block(block):
    """Validate block on tag or hex int
    """
    if isinstance(block, str):
        if block not in BLOCK_TAGS:
            raise ValueError('Invalid block tag.')
    elif isinstance(block, int):
        block = hex(block)
    else:
        raise ValueError('Invalid block type.')
    return block



import warnings
import asyncio

from ..utils import hex_to_dec, validate_block, ether_to_wei
from ..constants import BLOCK_TAG_LATEST


class EthMixin:

    @asyncio.coroutine
    def eth_protocolVersion(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_protocolversion

        :rtype: int
        """
        return hex_to_dec((yield from self.rpc_call('eth_protocolVersion')))

    @asyncio.coroutine
    def eth_syncing(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_syncing

        :return: sync status or false
        :rtype: dict or bool
        """
        return (yield from self.rpc_call('eth_syncing'))

    @asyncio.coroutine
    def eth_coinbase(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_coinbase

        :return: address
        :rtype: str
        """
        return (yield from self.rpc_call('eth_coinbase'))

    @asyncio.coroutine
    def eth_mining(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_mining

        :rtype: bool
        """
        return (yield from self.rpc_call('eth_mining'))

    @asyncio.coroutine
    def eth_hashrate(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_hashrate

        :return: hashrate
        :rtype: int
        """
        return hex_to_dec((yield from self.rpc_call('eth_hashrate')))

    @asyncio.coroutine
    def eth_gasPrice(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_gasprice

        :return: wei
        :rtype: int
        """
        return hex_to_dec((yield from self.rpc_call('eth_gasPrice')))

    @asyncio.coroutine
    def eth_accounts(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_accounts

        :return: accounts
        :rtype: list
        """
        return (yield from self.rpc_call('eth_accounts'))

    @asyncio.coroutine
    def eth_blockNumber(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_blocknumber

        :return: bnumber
        :rtype: int
        """
        return hex_to_dec((yield from self.rpc_call('eth_blockNumber')))

    @asyncio.coroutine
    def eth_getBalance(self, address, block=BLOCK_TAG_LATEST):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getbalance

        :param address: Account address
        :type address: str

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :return: wei
        :rtype: int
        """
        block = validate_block(block)
        return hex_to_dec((yield from self.rpc_call('eth_getBalance',
                                                    [address, block])))

    @asyncio.coroutine
    def eth_getStorageAt(self, address, position=0, block=BLOCK_TAG_LATEST):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getstorageat

        :param address: Storage address
        :type address: str

        :param position: Position in storage (optional)
        :type position: int

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :rtype: int
        """
        block = validate_block(block)
        return hex_to_dec((yield from self.rpc_call('eth_getStorageAt',
                                                    [address,
                                                     hex(position),
                                                     block])))

    @asyncio.coroutine
    def eth_getTransactionCount(self, address, block=BLOCK_TAG_LATEST):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_gettransactioncount

        :param address: Account address
        :type address: str

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :return: count
        :rtype: int
        """
        block = validate_block(block)
        return hex_to_dec((yield from self.rpc_call('eth_getTransactionCount',
                                                    [address, block])))

    @asyncio.coroutine
    def eth_getBlockTransactionCountByHash(self, bhash):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getblocktransactioncountbyhash

        :param bhash: Block hash
        :type bhash: str

        :return: count
        :rtype: int or None
        """
        response = yield from self.rpc_call('eth_getBlockTransactionCountByHash',
                                            [bhash])
        if response:
            return hex_to_dec(response)
        return response

    @asyncio.coroutine
    def eth_getBlockTransactionCountByNumber(self, block=BLOCK_TAG_LATEST):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getblocktransactioncountbynumber

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :return: count
        :rtype: int
        """
        block = validate_block(block)
        return hex_to_dec((yield from self.rpc_call('eth_getBlockTransactionCountByNumber',
                                                    [block])))

    @asyncio.coroutine
    def eth_getUncleCountByBlockHash(self, bhash):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getunclecountbyblockhash

        :param bhash: Block hash
        :type bhash: str

        :return: count
        :rtype: int or None
        """
        response = yield from self.rpc_call('eth_getUncleCountByBlockHash',
                                            [bhash])
        if response:
            return hex_to_dec(response)
        return response

    @asyncio.coroutine
    def eth_getUncleCountByBlockNumber(self, block=BLOCK_TAG_LATEST):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getunclecountbyblocknumber

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :return: count
        :rtype: int
        """
        block = validate_block(block)
        response = yield from self.rpc_call('eth_getUncleCountByBlockNumber',
                                            [block])
        if response:
            return hex_to_dec(response)
        return response

    @asyncio.coroutine
    def eth_getCode(self, address, block=BLOCK_TAG_LATEST):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getcode

        :param address: Address of contract
        :type address: str

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :return: code
        :rtype: str
        """
        block = validate_block(block)
        return (yield from self.rpc_call('eth_getCode', [address, block]))

    @asyncio.coroutine
    def eth_sign(self, address, data):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_sign

        :param address: Account address
        :type address: str

        :param data: Message to sign
        :type data: str

        :return: signature
        :rtype: str
        """
        return (yield from self.rpc_call('eth_sign', [address, data]))

    @asyncio.coroutine
    def eth_sendTransaction(self, from_, to=None, gas=None,
                            gas_price=None, value=None, data=None,
                            nonce=None):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_sendtransaction

        :param from_: From account address
        :type from_: str

        :param to: To account address (optional)
        :type to: str

        :param gas: Gas amount for current transaction (optional)
        :type gas: int

        :param gas_price: Gas price for current transaction (optional)
        :type gas_price: int

        :param value: Amount of ether to send (optional)
        :type value: int

        :param data: Additional data for transaction (optional)
        :type data: hex

        :param nonce: Unique nonce for transaction (optional)
        :type nonce: int

        :return: txhash
        :rtype: str
        """
        obj = {}
        obj['from'] = from_
        if to is not None:
            obj['to'] = to
        if gas is not None:
            obj['gas'] = hex(gas)
        if gas_price is not None:
            obj['gasPrice'] = hex(gas_price)
        if value is not None:
            obj['value'] = hex(ether_to_wei(value))
        if data is not None:
            obj['data'] = data
        if nonce is not None:
            obj['nonce'] = hex(nonce)

        return (yield from self.rpc_call('eth_sendTransaction', [obj]))

    @asyncio.coroutine
    def eth_sendRawTransaction(self, data):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_sendrawtransaction

        :param data: Signed transaction data
        :type data: str

        :return: txhash
        :rtype: str
        """
        return (yield from self.rpc_call('eth_sendRawTransaction', [data]))

    @asyncio.coroutine
    def eth_call(self, from_, to=None, gas=None,
                 gas_price=None, value=None, data=None,
                 block=BLOCK_TAG_LATEST):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_call

        :param from_: From account address
        :type from_: str

        :param to: To account address (optional)
        :type to: str

        :param gas: Gas amount for current transaction (optional)
        :type gas: int

        :param gas_price: Gas price for current transaction (optional)
        :type gas_price: int

        :param value: Amount of ether to send (optional)
        :type value: int

        :param data: Additional data for transaction (optional)
        :type data: hex

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :rtype: str
        """
        block = validate_block(block)
        obj = {}
        obj['from'] = from_
        if to is not None:
            obj['to'] = to
        if gas is not None:
            obj['gas'] = hex(gas)
        if gas_price is not None:
            obj['gasPrice'] = hex(gas_price)
        if value is not None:
            obj['value'] = hex(ether_to_wei(value))
        if data is not None:
            obj['data'] = data

        return (yield from self.rpc_call('eth_call', [obj, block]))

    @asyncio.coroutine
    def eth_estimateGas(self, from_, to=None, gas=None,
                        gas_price=None, value=None, data=None):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_estimategas

        :param from_: From account address
        :type from_: str

        :param to: To account address (optional)
        :type to: str

        :param gas: Gas amount for current transaction (optional)
        :type gas: int

        :param gas_price: Gas price for current transaction (optional)
        :type gas_price: int

        :param value: Amount of ether to send (optional)
        :type value: int

        :param data: Additional data for transaction (optional)
        :type data: hex

        :return: gas amount
        :rtype: int
        """
        obj = {}
        obj['from'] = from_
        if to is not None:
            obj['to'] = to
        if gas is not None:
            obj['gas'] = hex(gas)
        if gas_price is not None:
            obj['gasPrice'] = hex(gas_price)
        if value is not None:
            obj['value'] = hex(ether_to_wei(value))
        if data is not None:
            obj['data'] = data

        return hex_to_dec((yield from self.rpc_call('eth_estimateGas', [obj])))

    @asyncio.coroutine
    def eth_getBlockByHash(self, bhash, tx_objects=True):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getblockbyhash

        :param bhash: Block hash
        :type bhash: str

        :param tx_objects: Return txs full object (optional)
        :type tx_objects: bool

        :return: block
        :rtype: dict or None
        """
        result = yield from self.rpc_call('eth_getBlockByHash',
                                          [bhash, tx_objects])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getBlockByNumber(self, block=BLOCK_TAG_LATEST, tx_objects=True):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getblockbynumber

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :param tx_objects: Return txs full object (optional)
        :type tx_objects: bool

        :return: block
        :rtype: dict or None
        """
        block = validate_block(block)
        result = yield from self.rpc_call('eth_getBlockByNumber',
                                          [block, tx_objects])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getTransactionByHash(self, txhash):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_gettransactionbyhash

        :param txhash: Transaction hash
        :type txhash: str

        :return: transaction
        :rtype: dict or None
        """
        result = yield from self.rpc_call('eth_getTransactionByHash', [txhash])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getTransactionByBlockHashAndIndex(self, bhash, index=0):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_gettransactionbyblockhashandindex

        :param bhash: Block hash
        :type bhash: str

        :param index: Index position (optional)
        :type index: int
        """
        result = yield from self.rpc_call('eth_getTransactionByBlockHashAndIndex',
                                          [bhash, hex(index)])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getTransactionByBlockNumberAndIndex(self, block=BLOCK_TAG_LATEST,
                                                index=0):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_gettransactionbyblocknumberandindex

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :param index: Index position (optional)
        :type index: int

        :return: transaction
        :rtype: dict or None
        """
        block = validate_block(block)
        result = yield from self.rpc_call('eth_getTransactionByBlockNumberAndIndex',
                                          [block, hex(index)])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getTransactionReceipt(self, txhash):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_gettransactionreceipt

        :param txhash: Transaction hash
        :type txhash: str

        :return: transaction
        :rtype: dict or None
        """
        result = yield from self.rpc_call('eth_getTransactionReceipt',
                                          [txhash])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getUncleByBlockHashAndIndex(self, bhash, index=0):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getunclebyblockhashandindex

        :param bhash: Block hash
        :type bhash: str

        :param index: Index position (optional)
        :type index: int

        :return: block
        :rtype: dict or None
        """
        result = yield from self.rpc_call('eth_getUncleByBlockHashAndIndex',
                                          [bhash, hex(index)])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getUncleByBlockNumberAndIndex(self, block=BLOCK_TAG_LATEST,
                                          index=0):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getunclebyblocknumberandindex

        :param block: Block tag or number (optional)
        :type block: int or BLOCK_TAGS

        :param index: Index position (optional)
        :type index: int

        :return: block
        :rtype: dict or None
        """
        block = validate_block(block)
        result = yield from self.rpc_call('eth_getUncleByBlockNumberAndIndex',
                                          [block, hex(index)])
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def eth_getCompilers(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getcompilers

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('eth_getCompilers'))

    @asyncio.coroutine
    def eth_compileSolidity(self, code):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_compilesolidity

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('eth_compileSolidity', [code]))

    @asyncio.coroutine
    def eth_compileLLL(self, code):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_compilelll

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('eth_compileLLL', [code]))

    @asyncio.coroutine
    def eth_compileSerpent(self, code):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_compileserpent

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('eth_compileSerpent', [code]))

    @asyncio.coroutine
    def eth_newFilter(self, from_block=BLOCK_TAG_LATEST,
                      to_block=BLOCK_TAG_LATEST, address=None, topics=None):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_newfilter

        :param from_block: Block tag or number (optional)
        :type from_block: int or BLOCK_TAGS

        :param to_block: Block tag or number (optional)
        :type to_block: int or BLOCK_TAGS

        :param address: Contract address (optional)
        :type address: str

        :param topics: Topics (optional)
        :type topics: list

        :return: filter_id
        :rtype: str
        """
        obj = {
            'fromBlock': validate_block(from_block),
            'toBlock': validate_block(to_block),
            'address': address,
            'topics': topics
        }
        return (yield from self.rpc_call('eth_newFilter', [obj]))

    @asyncio.coroutine
    def eth_newBlockFilter(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_newblockfilter

        :return: filter_id
        :rtype: str
        """
        return (yield from self.rpc_call('eth_newBlockFilter'))

    @asyncio.coroutine
    def eth_newPendingTransactionFilter(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_newpendingtransactionfilter

        :return: filter_id
        :rtype: str
        """
        return (yield from self.rpc_call('eth_newPendingTransactionFilter'))

    @asyncio.coroutine
    def eth_uninstallFilter(self, filter_id):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_uninstallfilter

        :param filter_id: Id of created filter
        :type filter_id: str

        :return: success
        :rtype: bool
        """
        return (yield from self.rpc_call('eth_uninstallFilter', [filter_id]))

    @asyncio.coroutine
    def eth_getFilterChanges(self, filter_id):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getfilterchanges

        :param filter_id: Id of created filter
        :type filter_id: str

        :return: logs
        :rtype: list
        """
        return (yield from self.rpc_call('eth_getFilterChanges', [filter_id]))

    @asyncio.coroutine
    def eth_getFilterLogs(self, filter_id):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getfilterlogs

        :param filter_id: Id of created filter
        :type filter_id: str

        :return: logs
        :rtype: list
        """
        return (yield from self.rpc_call('eth_getFilterLogs', [filter_id]))

    @asyncio.coroutine
    def eth_getLogs(self, from_block=BLOCK_TAG_LATEST,
                    to_block=BLOCK_TAG_LATEST, address=None, topics=None):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getlogs

        :param from_block: Block tag or number (optional)
        :type from_block: int or BLOCK_TAGS

        :param to_block: Block tag or number (optional)
        :type to_block: int or BLOCK_TAGS

        :param address: Contract address (optional)
        :type address: str

        :param topics: Topics (optional)
        :type topics: list

        :return: logs
        :rtype: list
        """
        obj = {
            'fromBlock': validate_block(from_block),
            'toBlock': validate_block(to_block),
            'address': address,
            'topics': topics
        }
        result = yield from self.rpc_call('eth_getLogs', [obj])
        return result

    @asyncio.coroutine
    def eth_getWork(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_getwork

        :return: work
        :rtype: list
        """
        return (yield from self.rpc_call('eth_getWork'))

    @asyncio.coroutine
    def eth_submitWork(self, nonce, header, mix_digest):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_submitwork

        :param nonce: Nonce of work
        :type nonce: int

        :param header: Pow-hash header
        :type header: str

        :param mix_digest: Mix digest
        :type mix_digest: str

        :return: success
        :rtype: bool
        """
        return (yield from self.rpc_call('eth_submitWork',
                                         [nonce, header, mix_digest]))

    @asyncio.coroutine
    def eth_submitHashrate(self, hashrate, id_):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#eth_submithashrate

        :param hashrate: Hash rate
        :type hashrate: str

        :param id_: Random hex
        :type id_: str

        :return: success
        :rtype: bool
        """
        return (yield from self.rpc_call('eth_submitHashrate',
                                         [hex(hashrate), id_]))



import asyncio

from ..utils import hex_to_dec


class TxpoolMixin:

    @asyncio.coroutine
    def txpool_content(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#txpool_content

        :rtype: dict
        """
        result = yield from self.rpc_call('txpool_content')
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def txpool_inspect(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#txpool_inspect

        :rtype: dict
        """
        result = yield from self.rpc_call('txpool_inspect')
        # TODO: Update result response
        return result

    @asyncio.coroutine
    def txpool_status(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#txpool_status

        :rtype: dict
        """
        result = yield from self.rpc_call('txpool_status')
        return {k: hex_to_dec(v) for k, v in result.items()}



import warnings
import asyncio


class ShhMixin:

    @asyncio.coroutine
    def shh_version(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_version

        :rtype: float
        """
        return float((yield from self.rpc_call('shh_version')))

    @asyncio.coroutine
    def shh_post(self, from_=None, to=None, *, topics, payload, priority, ttl):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_post
        """
        # FIXME: Not working
        obj = {
            'from': from_,
            'to': to,
            'topics': topics,
            'payload': payload,
            'priority': hex(priority),
            'ttl': hex(ttl),
        }
        return float((yield from self.rpc_call('shh_post', [obj])))

    @asyncio.coroutine
    def shh_newIdentity(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_newidentity

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('shh_newIdentity'))

    @asyncio.coroutine
    def shh_hasIdentity(self, address):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_hasidentity

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('shh_hasIdentity', [address]))

    @asyncio.coroutine
    def shh_newGroup(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_newgroup

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        result = yield from self.rpc_call('shh_newGroup')
        return result

    @asyncio.coroutine
    def shh_addToGroup(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_addtogroup

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('shh_addToGroup'))

    @asyncio.coroutine
    def shh_newFilter(self, to=None, *, topics):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_newfilter

        DEPRECATED
        """
        obj = {
            'to': to,
            'topics': topics,
        }
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('shh_newFilter', [obj]))

    @asyncio.coroutine
    def shh_uninstallFilter(self, filter_id):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_uninstallfilter

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('shh_uninstallFilter', [filter_id]))

    @asyncio.coroutine
    def shh_getFilterChanges(self, filter_id):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_getfilterchanges

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('shh_getFilterChanges', [filter_id]))

    @asyncio.coroutine
    def shh_getMessages(self, filter_id):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#shh_getmessages

        DEPRECATED
        """
        warnings.warn('deprecated', DeprecationWarning)
        return (yield from self.rpc_call('shh_getMessages', [filter_id]))



import asyncio

from ..utils import hex_to_dec


class NetMixin:

    @asyncio.coroutine
    def net_version(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#net_version

        :rtype: str
        """
        return (yield from self.rpc_call('net_version'))

    @asyncio.coroutine
    def net_listening(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#net_listening

        :rtype: bool
        """
        return (yield from self.rpc_call('net_listening'))

    @asyncio.coroutine
    def net_peerCount(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#net_peercount

        :rtype: int
        """
        return hex_to_dec((yield from self.rpc_call('net_peerCount')))



import re
import os.path
import sys
import platform
from setuptools import setup, find_packages


uvloop = 'uvloop>=0.8.1'
install_requires = [
    'aiohttp>=2.2.5',
    'async-timeout>=1.4.0',
]
if platform.python_implementation() == 'CPython':
    install_requires.append('ujson')

PY_VER = sys.version_info

if PY_VER >= (3, 5) and sys.platform != "win32":
    install_requires.append(uvloop)
elif PY_VER >= (3, 4):
    pass
elif PY_VER >= (3, 3):
    install_requires.append('asyncio>=3.4.3')
else:
    raise RuntimeError("aioethereum doesn't support Python version prior 3.3")


def read(*parts):
    with open(os.path.join(*parts), 'rt') as f:
        return f.read().strip()


def read_version():
    regexp = re.compile(r"^__version__\W*=\W*'([\d.abrc]+)'")
    init_py = os.path.join(os.path.dirname(__file__),
                           'aioethereum', '__init__.py')
    with open(init_py) as f:
        for line in f:
            match = regexp.match(line)
            if match is not None:
                return match.group(1)
        else:
            raise RuntimeError('Cannot find version in '
                               'aioethereum/__init__.py')


classifiers = [
    'License :: OSI Approved :: MIT License',
    'Development Status :: 4 - Beta',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Operating System :: POSIX',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries',
    'Framework :: AsyncIO',
]

setup(
    name='aioethereum',
    version=read_version(),
    description=("Ethereum RPC client library for Python asyncio (PEP 3156)"),
    long_description="\n\n".join((read('README.rst'), read('CHANGES.rst'))),
    classifiers=classifiers,
    platforms=["POSIX"],
    author="Bogdan Kurinnyi",
    author_email="bogdankurinniy.dev1@gmail.com",
    url="https://github.com/DeV1doR/aioethereum",
    license="MIT",
    packages=find_packages(exclude=["tests"]),
    install_requires=install_requires,
    include_package_data=True,
)



import asyncio
import binascii

from ..utils import add_0x


class Web3Mixin:

    @asyncio.coroutine
    def web3_clientVersion(self):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#web3_clientversion

        :return: version
        :rtype: str
        """
        return (yield from self.rpc_call('web3_clientVersion'))

    @asyncio.coroutine
    def web3_sha3(self, data):
        """https://github.com/ethereum/wiki/wiki/JSON-RPC#web3_sha3

        :param data: Data for hashing
        :type data: str

        :return: Keccak-256
        :rtype: str
        """
        data = add_0x(binascii.hexlify(str(data).encode('utf-8')))
        return (yield from self.rpc_call('web3_sha3', [data]))



import asyncio


class MinerMixin:

    @asyncio.coroutine
    def miner_setExtra(self, data):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#miner_setextra

        :param data: Extra data a miner can include when miner blocks
        :type data: str

        :rtype: bool
        """
        return (yield from self.rpc_call('miner_setExtra', [data]))

    @asyncio.coroutine
    def miner_setGasPrice(self, number):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#miner_setgasprice

        :param number: Minimal accepted gas price when mining transactions
        :type number: int

        :rtype: bool
        """
        return (yield from self.rpc_call('miner_setGasPrice', [hex(number)]))

    @asyncio.coroutine
    def miner_start(self, number):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#miner_start

        :param number: CPU mining process with the given number of threads
        :type number: int

        :rtype: None
        """
        return (yield from self.rpc_call('miner_start', [number]))

    @asyncio.coroutine
    def miner_stop(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#miner_stop

        :rtype: bool
        """
        return (yield from self.rpc_call('miner_stop'))

    @asyncio.coroutine
    def miner_setEtherBase(self, address):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#miner_setetherbase

        NOT AVAILABLE
        """
        return (yield from self.rpc_call('miner_setEtherBase', [address]))



import asyncio


class DebugMixin:

    @asyncio.coroutine
    def debug_backtraceAt(self, filename_line):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_backtraceat

        :param filename_line: Filename and its line for debugging
        :type filename_line: str

        :rtype: None
        """
        return (yield from self.rpc_call('debug_backtraceAt', [filename_line]))

    @asyncio.coroutine
    def debug_blockProfile(self, file, seconds):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_blockprofile

        :param file: Log file path
        :type file: str

        :param seconds: Seconds for profile capturing
        :type seconds: int

        :rtype: None
        """
        return (yield from self.rpc_call('debug_blockProfile', [file, seconds]))

    @asyncio.coroutine
    def debug_cpuProfile(self, file, seconds):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_cpuprofile

        :param file: Log file path
        :type file: str

        :param seconds: Seconds for profile capturing
        :type seconds: int

        :rtype: None
        """
        return (yield from self.rpc_call('debug_cpuProfile', [file, seconds]))

    @asyncio.coroutine
    def debug_dumpBlock(self, number):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_dumpblock

        :param number: Block number
        :type number: int

        :return: accounts
        :rtype: dict
        """
        return (yield from self.rpc_call('debug_dumpBlock', [hex(number)]))

    @asyncio.coroutine
    def debug_gcStats(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_gcstats

        :return: https://golang.org/pkg/runtime/debug/#GCStats
        :rtype: dict
        """
        return (yield from self.rpc_call('debug_gcStats'))

    @asyncio.coroutine
    def debug_getBlockRlp(self, number):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_getblockrlp

        :param number: Block number
        :type number: int

        :return: RLP
        :rtype: str
        """
        return (yield from self.rpc_call('debug_getBlockRlp', [number]))

    @asyncio.coroutine
    def debug_goTrace(self, file, seconds):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_gotrace

        :param file: Log file path
        :type file: str

        :param seconds: Seconds for profile capturing
        :type seconds: int

        :rtype: None
        """
        return (yield from self.rpc_call('debug_goTrace', [file, seconds]))

    @asyncio.coroutine
    def debug_memStats(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_memstats

        :return: https://golang.org/pkg/runtime/debug/#MemStats
        :rtype: dict
        """
        return (yield from self.rpc_call('debug_memStats'))

    @asyncio.coroutine
    def debug_seedHash(self, number):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_seedhash

        :param number: Block number
        :type number: int

        :return: hash
        :rtype: str or error
        """
        return (yield from self.rpc_call('debug_seedHash', [number]))

    @asyncio.coroutine
    def debug_setHead(self, number):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_sethead

        :param number: Block number
        :type number: int

        :rtype: None
        """
        return (yield from self.rpc_call('debug_setHead', [hex(number)]))

    @asyncio.coroutine
    def debug_setBlockProfileRate(self, rate):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_sethead

        :param rate: Rate of goroutine block (samples/sec)
        :type rate: int

        :rtype: None
        """
        return (yield from self.rpc_call('debug_setBlockProfileRate', [rate]))

    @asyncio.coroutine
    def debug_stacks(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_stacks

        :rtype: str
        """
        return (yield from self.rpc_call('debug_stacks'))

    @asyncio.coroutine
    def debug_startCPUProfile(self, file):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_startcpuprofile

        :param file: Log file path
        :type file: str

        :rtype: None
        """
        return (yield from self.rpc_call('debug_startCPUProfile', [file]))

    @asyncio.coroutine
    def debug_startGoTrace(self, file):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_startgotrace

        :param file: Log file path
        :type file: str

        :rtype: None
        """
        return (yield from self.rpc_call('debug_startGoTrace', [file]))

    @asyncio.coroutine
    def debug_stopCPUProfile(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_stopcpuprofile

        :rtype: None
        """
        return (yield from self.rpc_call('debug_stopCPUProfile'))

    @asyncio.coroutine
    def debug_stopGoTrace(self):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_stopgotrace

        :rtype: None
        """
        return (yield from self.rpc_call('debug_stopGoTrace'))

    @asyncio.coroutine
    def debug_traceBlock(self, block_rlp):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_traceblock

        # TODO: Add more description for params

        :rtype: dict
        """
        return (yield from self.rpc_call('debug_traceBlock', [block_rlp]))

    @asyncio.coroutine
    def debug_traceBlockByNumber(self, number):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_traceblockbynumber

        :param number: Block number
        :type number: int

        :rtype: dict
        """
        return (yield from self.rpc_call('debug_traceBlockByNumber',
                                         [hex(number)]))

    @asyncio.coroutine
    def debug_traceBlockByHash(self, bhash):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_traceblockbyhash

        :param bhash: Block hash
        :type bhash: str

        :rtype: dict
        """
        return (yield from self.rpc_call('debug_traceBlockByHash', [bhash]))

    @asyncio.coroutine
    def debug_traceBlockFromFile(self, file):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_traceblockfromfile

        :param file: Log file path
        :type file: str

        :rtype: dict
        """
        return (yield from self.rpc_call('debug_traceBlockFromFile', [file]))

    @asyncio.coroutine
    def debug_traceTransaction(self, txhash):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_tracetransaction

        :param txhash: Transaction hash
        :type txhash: str

        :rtype: dict
        """
        return (yield from self.rpc_call('debug_traceTransaction', [txhash]))

    @asyncio.coroutine
    def debug_verbosity(self, level):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_verbosity

        :param level: Debug lvl
        :type level: int

        :rtype: None
        """
        return (yield from self.rpc_call('debug_verbosity', [level]))

    @asyncio.coroutine
    def debug_vmodule(self, pattern):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_vmodule

        :param pattern: Logging verbosity pattern
        :type pattern: str

        :rtype: None
        """
        return (yield from self.rpc_call('debug_vmodule', [pattern]))

    @asyncio.coroutine
    def debug_writeBlockProfile(self, file):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_writeblockprofile

        :param file: Log file path
        :type file: str

        :rtype: None
        """
        return (yield from self.rpc_call('debug_writeBlockProfile', [file]))

    @asyncio.coroutine
    def debug_writeMemProfile(self, file):
        """https://github.com/ethereum/go-ethereum/wiki/Management-APIs#debug_writememprofile

        :param file: Log file path
        :type file: str

        :rtype: None
        """
        return (yield from self.rpc_call('debug_writeMemProfile', [file]))



from .client import (
    AsyncIOHTTPClient,
    AsyncIOIPCClient,
    BaseAsyncIOClient,
    create_ethereum_client
)


__version__ = '0.2.1'

__all__ = [
    'AsyncIOHTTPClient',
    'AsyncIOIPCClient',
    'BaseAsyncIOClient',
    'create_ethereum_client',
]
